@@ -148,6 +148,11 @@ class GBMFitter:
 
         if self.use_rounders:
             rounder = OptRounder()
+
+        obj = model_data['obj'] if 'obj' in model_data else None  # Objective function
+        num_boost_rounds = model_data['n'] if 'n' in model_data else 5000  # Num rounds
+        early_stopping = model_data['es'] if 'es' in model_data else None  # Early Stopping
+
             
         # Train and predict train and validation sets
         if library == 'CB':
@@ -156,17 +161,20 @@ class GBMFitter:
                                         cat_features=self._get_categorical_index(X_train, categorical)),
                          params=model_data['params'],
                          logging_level='Silent',
-                         num_boost_round=model_data['n'])
+                         num_boost_round=num_boost_rounds,
+                         early_stopping=early_stopping,
+                         eval_set=cb.Pool(data=X_val,
+                                          label=y_val,
+                                          cat_features=self._get_categorical_index(X_train, categorical)) if early_stopping else None)
             y_pred_train = m.predict(X_train)
             y_pred_val = m.predict(X_val)
 
         if library == 'XGB':
-            obj = None
-            if 'obj' in model_data:
-                obj = model_data['obj']
             m = xgb.train(params=model_data['params'],
                           dtrain=xgb.DMatrix(X_train, y_train),
-                          num_boost_round=model_data['n'],
+                          num_boost_round=num_boost_rounds,
+                          early_stopping_rounds=early_stopping,
+                          evals=[(xgb.DMatrix(X_train, y_train), 'Validation')],
                           obj=obj)
             y_pred_train = m.predict(xgb.DMatrix(X_train))
             y_pred_val = m.predict(xgb.DMatrix(X_val))
@@ -177,17 +185,21 @@ class GBMFitter:
             m.load_model('xgb_temp')
 
         if library == 'LGB':
-            obj = None
-            if 'obj' in model_data:
-                obj = model_data['obj']
             m = lgb.train(params=model_data['params'],
                           train_set=lgb.Dataset(X_train,
                                                 label=y_train,
                                                 free_raw_data=False),
                           categorical_feature=self._get_categorical_index(X_train,
                                                                           categorical) if categorical else 'auto',
-                          num_boost_round=model_data['n'],
-                          fobj=obj)
+                          num_boost_round=num_boost_rounds,
+                          fobj=obj,
+                          early_stopping_rounds=early_stopping,
+                          valid_sets=[lgb.Dataset(X_val,
+                                                 label=y_val,
+                                                 free_raw_data=False)] if early_stopping else None,
+                          valid_names=['Validation'] if early_stopping else None
+                          )
+            # Predict validation dataset to get train result and OOF validation result
             y_pred_train = m.predict(X_train)
             y_pred_val = m.predict(X_val)
 
@@ -197,7 +209,7 @@ class GBMFitter:
             y_pred_train = rounder.predict(y_pred_train)
             y_pred_val = rounder.predict(y_pred_val)
 
-        # Evaluate Train
+        # Calculate metrics
         for metric in self.metrics:
             if metric == 'rmse':
                 train_metric = mt.mean_squared_error(y_train, y_pred_train, squared=False)
@@ -205,6 +217,7 @@ class GBMFitter:
             else:
                 train_metric = metric(y_train, y_pred_train)
                 val_metric = metric(y_val, y_pred_val)
+
         # acc_train = accuracy_score(y_train, y_pred_train)
         # f1_train = f1_score(y_train, y_pred_train, average='macro')
 
