@@ -56,11 +56,13 @@ def _update_params(hyperopt_params, params):
     return hyperopt_params
 
 
-def _save_json(data, path):
+def _save_json(data, path, verbose=1):
     if path[path.rfind('.'):] != '.json':
         path = path + '.json'
     with open(path, 'w') as fp:
         json.dump(data, fp)
+    if verbose > 0:
+        print('Best parameters saved to '+path)
 
 
 def opt_catboost(X_train,
@@ -75,7 +77,7 @@ def opt_catboost(X_train,
                  max_rounds=5000,
                  seed=0,
                  early_stopping=20,
-                 figsize=(15,6),
+                 figsize=(15, 6),
                  savepath=None):
     """
     Performs n_trials using Hyperopt to obtain the best catboost parameters
@@ -348,11 +350,12 @@ def opt_lightgbm(X_train,
     HYPEROPT_PARAMS = _update_params(HYPEROPT_PARAMS, params)
     INTEGER_PARAMS_LIST = ['max_depth', 'seed', 'bagging_freq','num_leaves','min_data_in_leaf']
 
-    dataset = lgb.Dataset(X_train, label=y_train, categorical_feature =cat_features)
 
     def objective(params):
         start = time.time()
 
+        dataset = lgb.Dataset(X_train, label=y_train, categorical_feature=cat_features)
+
         # Control Integer Params don't go float
         for par_name in INTEGER_PARAMS_LIST:
             params[par_name] = int(np.round(params[par_name]))
